@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2008 Keio University
+#     Copyright (C) 2001-2009 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: G::IO::Handler.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -249,7 +249,12 @@ sub before_startcodon {
     if ($this->{$object}->{direction} eq 'complement'){
 	return complement(substr($this->{SEQ}, $this->{$object}->{end}, $length));
     }else{
-	return substr($this->{SEQ}, $this->{$object}->{start} - 1 - $length, $length);
+	my $start = $this->{$object}->{start} - 1 - $length;
+	if ($start < 0){
+	    $start = 0;
+	    $length = $this->{$object}->{start} - 1;
+	}
+	return substr($this->{SEQ}, $start, $length);
     }
 }
 
@@ -259,7 +264,12 @@ sub after_startcodon {
     my $length = shift || 100;
 
     if ($this->{$object}->{direction} eq 'complement'){
-	return complement(substr($this->{SEQ}, $this->{$object}->{end} - 1 - 3 - $length + 1, $length));
+	my $start = $this->{$object}->{end} - 1 - 3 - $length + 1;
+	if($start < 0){
+	    $start = 0;
+	    $length = $this->{$object}->{end} - 3;
+	}
+	return complement(substr($this->{SEQ}, $start, $length));
     }else{
 	return substr($this->{SEQ}, $this->{$object}->{start} + 3 - 1, $length);
     }
@@ -273,7 +283,12 @@ sub before_stopcodon {
     if ($this->{$object}->{direction} eq 'complement'){
 	return complement(substr($this->{SEQ}, $this->{$object}->{start} + 3 - 1, $length));
     }else{
-	return substr($this->{SEQ}, $this->{$object}->{end} - 3 - 1 - $length + 1, $length);
+	my $start = $this->{$object}->{end} - 3 - 1 - $length + 1;
+	if($start < 0){
+	    $start = 0;
+	    $length = $this->{$object}->{end} - 3;
+	}
+	return substr($this->{SEQ}, $start, $length);
     }
 }
 
@@ -283,7 +298,12 @@ sub after_stopcodon {
     my $length = shift || 100;
 
     if ($this->{$object}->{direction} eq 'complement'){
-	return complement(substr($this->{SEQ}, $this->{$object}->{start} - 1 - $length, $length));
+	my $start = $this->{$object}->{start} - 1 - $length;
+	if($start < 0){
+	    $start = 0;
+	    $length = $this->{$object}->{start};
+	}
+	return complement(substr($this->{SEQ}, $start, $length));
     }else{
 	return substr($this->{SEQ}, $this->{$object}->{end} +1 - 1, $length);
     }
@@ -294,20 +314,37 @@ sub startcodon {
 }
 
 sub stopcodon {
-    return substr($_[0]->get_geneseq($_[0]), -3, 3);
+    return substr($_[0]->get_geneseq($_[1]), -3, 3);
 }
 
 sub pos2feature {
     my $this = shift;
     my $pos = shift;
+    my $pos2 = shift;
 
-    foreach my $feat ($this->feature()){
-	next if ($feat eq 'FEATURE0');
+    if(length($pos2) > 1){
+	my @genes = ();
 
-	if ($pos >= $this->{$feat}->{start} && $pos <= $this->{$feat}->{end}){
-	    return $feat;
-	}elsif ($pos < $this->{$feat}->{start}){
-	    return '';
+	foreach my $feat ($this->feature()){
+	    next if ($feat eq 'FEATURE0');
+
+	    if ($this->{$feat}->{end} >= $pos && $pos2 >= $this->{$feat}->{start}){
+		push(@genes, $feat);
+	    }elsif ($pos2 < $this->{$feat}->{start}){
+		return @genes;
+	    }
+	}
+
+    }else{
+
+	foreach my $feat ($this->feature()){
+	    next if ($feat eq 'FEATURE0');
+
+	    if ($pos >= $this->{$feat}->{start} && $pos <= $this->{$feat}->{end}){
+		return $feat;
+	    }elsif ($pos < $this->{$feat}->{start}){
+		return '';
+	    }
 	}
     }
 }
@@ -333,12 +370,26 @@ sub feature2id {
 sub pos2gene {
     my $this = shift;
     my $pos = shift;
+    my $pos2 = shift;
 
-    foreach my $feat ($this->cds()){
-	if ($pos >= $this->{$feat}->{start} && $pos <= $this->{$feat}->{end}){
-	    return $feat;
-	}elsif ($pos < $this->{$feat}->{start}){
-	    return '';
+    if(length($pos2) > 1){
+	my @genes = ();
+
+	foreach my $feat ($this->cds()){
+	    if ($this->{$feat}->{end} >= $pos && $pos2 >= $this->{$feat}->{start}){
+		push(@genes, $feat);
+	    }elsif ($pos2 < $this->{$feat}->{start}){
+		return @genes;
+	    }
+	}
+
+    }else{
+	foreach my $feat ($this->cds()){
+	    if ($pos >= $this->{$feat}->{start} && $pos <= $this->{$feat}->{end}){
+		return $feat;
+	    }elsif ($pos < $this->{$feat}->{start}){
+		return '';
+	    }
 	}
     }
 }
