@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2008 Keio University
+#     Copyright (C) 2001-2009 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: PatSearch.pm,v 1.3 2002/07/30 17:40:56 gaou Exp $
@@ -323,6 +323,7 @@ sub palindrome {
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
     
   History:
+    20081124-01 fixed a bug for handling -type option
     20071028-01 fixed a bug that did not properly search the complement 
     20071023-01 updated to include dif consensus of other organisms
     20060711-01 initial posting
@@ -333,7 +334,7 @@ sub palindrome {
 sub find_dif {
     opt_default(-type=>'ecoli');
     my @argv = opt_get(@_);
-    my $type = lc opt_val('ecoli');
+    my $type = lc opt_val('type');
     my $dif = 'ggtgcgcataatgtatattatgttaaat';
     $dif    = 'rntkcgcataatgtatattatgttaaat' if ($type eq 'proteobacteria');
     $dif    = 'acttcctagaatatatattatgtaaact' if ($type eq 'bsub');
